﻿//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  Werterfassung als Ringpufferarchiv (für SInt08 Werte)                               //
//  _________________________________                                                   //
//                                                                                      //
//  Copyright (c) 2016 Reimar Grasbon													//
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  This software is subjected to the FreeBSD License. Redistribution and use in source //
//  and binary forms, with or without modification, are permitted provided that the     //
//  following conditions are met:                                                       //
//                                                                                      //
//  1. Redistributions of source code must retain the above copyright notice, this list //
//     of conditions and the following disclaimer.                                      //
//                                                                                      //
//  2. Redistributions in binary form must reproduce the above copyright notice, this   //
//     list of conditions and the following disclaimer in the documentation and/or      //
//     other materials provided with the distribution.                                  //
//                                                                                      //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY //
//  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED           //
//  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  //
//  IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,    //
//  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT  //
//  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR  //
//  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,   //
//  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)  //
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE          //
//  POSSIBILITY OF SUCH DAMAGE.                                                         //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////

#include "Conf.h"
#include "Logger.h"

//////////////////////////////////////////////////////////////////////////////////////////
//  Definitionen  ////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
#define INVALID_LOG  CFG_TEMPERATURE_INVALID
#define LOG_CAPACITY CFG_TEMPERATURE_LOG_CAPACITY

//////////////////////////////////////////////////////////////////////////////////////////
//  Daten  ///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
static SInt08  ringBuffer[LOG_CAPACITY];
static UInt16  recent = 0;

//////////////////////////////////////////////////////////////////////////////////////////
//  Funktionen  //////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////
//	LOGGER_Capacity(): Liefert die maximale Anzahl an Einträgen im Ringpuffer.          //
//  -----------------                                                                   //
//																						//
//	Eingabe: -                                                                          //
//	Ausgabe: Kapazität des Ringpuffers.                                                 //
//////////////////////////////////////////////////////////////////////////////////////////
UInt16
LOGGER_Capacity(void)
{
	return CARDINALITY(ringBuffer);
}

//////////////////////////////////////////////////////////////////////////////////////////
//	LOGGER_Clear(): Überschreibt die komplette Datenaufzeichnung mit dem Wert, der      //
//  --------------  in <INVALID_LOG> hinterlegt ist.                                    //
//																						//
//	Eingabe: · data: Wert den alle Datenfelder erhalten sollen.							//
//	Ausgabe: -																			//
//////////////////////////////////////////////////////////////////////////////////////////
void
LOGGER_Clear(void)
{
	for (UInt16 i=0; i<CARDINALITY(ringBuffer); i++) ringBuffer[i] = INVALID_LOG;
	recent = 0;
}

//////////////////////////////////////////////////////////////////////////////////////////
//	LOGGER_Log(): Zeichnet einen neuen Wert auf und verdrängt damit den am weitesten	//
//  ------------  zurückliegenden Wert. Der	neue Wert erhält Position 0.				//
//																						//
//	Eingabe: · data: Neuer aufzuzeichnender Wert.										//
//	Ausgabe: -																			//
//////////////////////////////////////////////////////////////////////////////////////////
void
LOGGER_Log(SInt08 data)
{
	recent++;
	if (recent == CARDINALITY(ringBuffer)) recent = 0;
	ringBuffer[recent] = data;
}

//////////////////////////////////////////////////////////////////////////////////////////
//	LOGGER_At(): Liefert den Wert an Position <position> der Werterfassung. Der zuletzt //
//  -----------  aufgezeichnete Wert steht an Position 0.								//
//																						//
//	Eingabe: · position: Auswahl des Wertes: 0 .. LOGGER_Capacity()-1.				    //
//	Ausgabe: Der aufgezeichnete Wert an Position <position>.							//
//////////////////////////////////////////////////////////////////////////////////////////
SInt08
LOGGER_At(UInt16 position)
{
	UInt16 index;
	if (position > recent) index = CARDINALITY(ringBuffer) - position + recent;
	else                   index = recent - position;
	return ringBuffer[index];
}

//////////////////////////////////////////////////////////////////////////////////////////

